package com.seitenbau.testing.recorderexampleusage;

import java.util.List;

import com.seitenbau.testing.service.RepositoryService;
import com.seitenbau.testing.shared.config.RecorderConfiguration;
import com.seitenbau.testing.shared.model.InvocationBase;
import com.seitenbau.testing.shared.model.InvocationMethod;
import com.seitenbau.testing.shared.model.RecordedMethodConfiguration;
import com.seitenbau.testing.shared.model.RecordedTest;
import com.seitenbau.testing.shared.model.RecordedTestCase;



public class RunRetrieve {
	private static int ident = 0;

	public static void main(String[] args) {
		RecorderConfiguration rcfg = ConfigurationBuilder.build();
		List<RecordedMethodConfiguration> configs = 
				RepositoryService.getRecordedMethodConfigurations(
						rcfg
					);
		
		for (RecordedMethodConfiguration recordConfig : configs) 
		{
			printRecordConfig(recordConfig);
			
			List<RecordedTestCase> cases = recordConfig.getRecordedTestCases();
			incIdent();
			for (RecordedTestCase recordedTestCase : cases) 
			{
				List<RecordedTest> tests = recordedTestCase.getRecordedTests();
				
				printRecordedTestCase(recordedTestCase);
				
				incIdent();
				for (RecordedTest recordedTest : tests) 
				{
					printRecordedTest(recordedTest);
					
				}
				decIdent();
			}
			decIdent();
		}
		
	}

	private static void printRecordedTest(RecordedTest recordedTest) {
		printCallToMethods(recordedTest);
		
		List<InvocationBase> trace = recordedTest.getTrace();
		println("*** contains " + trace.size() + " further sub-call ***");
		
		incIdent();
		for (InvocationBase callToBase : trace) {
			printCallToBase(callToBase);
		}
		decIdent();
	}

	private static void printCallToMethods(InvocationMethod recordedTest) {
		println(recordedTest.getClassName() + "::" + recordedTest.getMethodName() + "(");
		@SuppressWarnings("unused")
		String[] argT = recordedTest.getArgumentTypes();
//		Object[] obj=(Object[]) recordedTest.getArgumentValues().();
//		incIdent();
//		for(int i=0;i<argT.length;++i) {
//			println( argT[i] + "={" + obj[i] +"}");
//		}
//		decIdent();
//		println(" );");
//		if(recordedTest.isVoidMethod()) {
//			println("returned: nothing (void method)");
//		} else {
//			println("returned: " + recordedTest.getReturnType()+ "={"+recordedTest.getReturnValue().makeObject()+"}");
//		}
	}

	private static void printCallToBase(InvocationBase callToBase) {
		incIdent();
		if(callToBase instanceof InvocationMethod) {
			println("* call to Method:");
			incIdent();
			printCallToMethods((InvocationMethod) callToBase);
			println("");
			decIdent();
		}
		decIdent();
	}

	private static void printRecordedTestCase( RecordedTestCase recordedTestCase) {
		println("Recorded Test Case : \"" + recordedTestCase.getName()+"\"");
		println("Contains " + recordedTestCase.getRecordedTests().size() +" Calls to target:");
	}

	private static void printRecordConfig(RecordedMethodConfiguration recordConfig) {
		println(
				recordConfig.getClassPackage()+"."
				+recordConfig.getClassName()+"::"
				+recordConfig.getMethodName()
			);
	}
	
	private static void decIdent() {
		ident--;
	}

	private static void incIdent() {
		ident++;
	}

	private static void println(String string) {
		for(int i=0;i<ident;++i) {
			System.out.print("  ");
		}
		System.out.println(string);
	}

	@SuppressWarnings("unused")
	private static void print(String string) {
		for(int i=0;i<ident;++i) {
			System.out.print("  ");
		}
		System.out.print(string);
	}
}
